unit ClassFalling;

interface

uses
  Classes, Contnrs, ClassFallingSlice;

type

{ \hqU }
  TFalling = class(TObjectList)
  private
    FOnAllFull: TNotifyEvent;
    FFrequency: Byte;
    procedure SetFrequency(Value: Byte);
  protected
    function GetItem(Index: Integer): TFallingSlice;
    procedure SetItem(Index: Integer; Value: TFallingSlice);
  public
    constructor Create;
    procedure Add(AObject: TFallingSlice);
    procedure FallingStep;
    function IsAllFull: Boolean;
    procedure Reset;
    procedure Dump;
    property Frequency: Byte read FFrequency write SetFrequency default 50;
    property Items[Index: Integer]: TFallingSlice read GetItem write SetItem;
    property OnAllFull: TNotifyEvent read FOnAllFull write FOnAllFull;
  end;

implementation

uses ClassDefinitions;

{ TFalling }

{==============================================================================}
{   W[@Item                                                               }
{==============================================================================}
procedure TFalling.Add(AObject: TFallingSlice);
begin
  inherited Add(AObject);
end;

{==============================================================================}
{   U@                                                               }
{==============================================================================}
procedure TFalling.FallingStep;
var
  i: Integer;
begin
  for i := 0 to Count-1 do
  begin
    Items[i].Falling;
    // YOF, _Bz
    if IsAllFull then
    begin
      // YwƥHandler, Ĳo
      if Assigned(FOnAllFull) then FOnAllFull(Self);
      break;
    end;
  end;
end;

function TFalling.GetItem(Index: Integer): TFallingSlice;
var
  o: TObject;
begin
  o := inherited GetItem(Index);
  Result := (o as TFallingSlice);
end;

procedure TFalling.SetItem(Index: Integer; Value: TFallingSlice);
begin
  inherited SetItem(Index, Value);
end;

{==============================================================================}
{   ᳣ﺡF?                                                        }
{==============================================================================}
function TFalling.IsAllFull: Boolean;
var
  i:    Integer;
begin
  Result := True;
  for i:=0 to Count-1 do
  begin
    if not Items[i].IsStackFull then
    begin
      Result := False;
      break;
    end;
  end;
end;

{==============================================================================}
{   Mų                                                                   }
{==============================================================================}
procedure TFalling.Reset;
var
  i:    Integer;
begin
  for i:=0 to Count-1 do
    GetItem(i).Reset;
end;

{==============================================================================}
{   HLogLXӤe                                                          }
{==============================================================================}
procedure TFalling.Dump;
var
  i:    Integer;
begin
  for i:=0 to Count-1 do
    GetItem(i).Dump;
end;

procedure TFalling.SetFrequency(Value: Byte);
var
  i:    Integer;
begin
  if (Value = FFrequency) then Exit;
  if (Value > FREQUENCY_MAX) then
    FFrequency := FREQUENCY_MAX
  else if (Value < FREQUENCY_MIN) then
    FFrequency := FREQUENCY_MIN
  else
    FFrequency := Value;

  for i:=0 to Count-1 do
    GetItem(i).Frequency := FFrequency;
end;

constructor TFalling.Create;
begin
  inherited;
  FFrequency := 50;
end;

end.
