unit ClassFallingSlice;

interface

uses
  SysUtils, Classes, ClassDefinitions;

type

{ W: @qU }
  TFallingSlice = class(TObject)
  private
    FArray: array of Integer;
    FCapacity: Integer;
    FStackHeight: Integer;
    FOnLog: TLogEvent;
    FLoBound: Integer;
    FHiBound: Integer;
    FID: Integer;
    FFrequency: Byte;
    function GetItem(Index: Integer): Integer;
    procedure SetItem(Index: Integer; Value: Integer);
    procedure Generate;
    procedure CalcStackHeight;
    procedure Log(Msg: String);
    procedure SetFrequency(Value: Byte);
  public
    constructor Create(Capacity: Integer = 100);
    destructor Destroy; override;
    procedure Falling;
    function IsStackFull: Boolean;
    procedure Reset;
    procedure Dump;
    property Capacity: Integer read FCapacity;
    property Frequency: Byte read FFrequency write SetFrequency default 50;
    property ID: Integer read FID write FID;
    property Items[Index: Integer]: Integer read GetItem write SetItem;
    property StackHeight: Integer read FStackHeight;
    property OnLog: TLogEvent read FOnLog write FOnLog;
  end;

implementation

{ TFallingSlice }

{==============================================================================}
{   غcl                                                                     }
{==============================================================================}
constructor TFallingSlice.Create(Capacity: Integer);
begin
  inherited Create;
  FCapacity := Capacity;
  FStackHeight := 0;
  FFrequency := 50;
  SetLength(FArray, Capacity);
  FLoBound := Low(FArray);
  FHiBound := High(FArray);
end;

{==============================================================================}
{   Ѻcl                                                                     }
{==============================================================================}
destructor TFallingSlice.Destroy;
begin
  FArray := nil;
  inherited;
end;

{==============================================================================}
{   ˬdO_F                                                             }
{==============================================================================}
function TFallingSlice.IsStackFull: Boolean;
begin
  Result := (FStackHeight > FHiBound);
end;

{==============================================================================}
{   pثeᰪ                                                           }
{==============================================================================}
procedure TFallingSlice.CalcStackHeight;
var
  i:    Integer;
begin
  for i:=FLoBound to FHiBound do
  begin
    if FArray[i] = SKY then
    begin
      FStackHeight := i;
      break;
    end
    else
      FStackHeight := i+1;
  end;
end;

{==============================================================================}
{   U@                                                               }
{==============================================================================}
procedure TFallingSlice.Falling;
var
  i:    Integer;
begin
  // wgF? YOhBz
  if IsStackFull then Exit;
  for i:=FStackHeight+1 to FHiBound do
    FArray[i-1] := FArray[i];
  FArray[FHiBound] := SKY;
  // pثeᰪ
  CalcStackHeight;
  // b̤Wͤ@ӷs
  Generate;
end;

{==============================================================================}
{   b̤Wͤ@ӳ                                                       }
{==============================================================================}
procedure TFallingSlice.Generate;
begin
  // wgF? YOhBz
  if IsStackFull then Exit;
  if (Random(256) mod (FREQUENCY_MAX - FFrequency + 1) = 0) then
  begin
    // קK"@@"
    if (FArray[FHiBound-1] <> SNOW) or
      ((FArray[FHiBound-1] = SNOW) and (FStackHeight = FHiBound)) then // uѤU@ӪŶ
      FArray[FHiBound] := SNOW;
    // pثeᰪ
    CalcStackHeight;
  end
  else
    FArray[FHiBound] := SKY;
end;

function TFallingSlice.GetItem(Index: Integer): Integer;
begin
  if (Index < FLoBound) or (Index > FHiBound) then
    raise Exception.CreateFmt('Out of Index(%d)', [Index]);
  Result := FArray[Index];
end;

procedure TFallingSlice.SetItem(Index: Integer; Value: Integer);
begin
  if (Index < FLoBound) or (Index > FHiBound) then
    raise Exception.CreateFmt('Out of Index(%d)', [Index]);
  FArray[Index] := Value;
end;

{==============================================================================}
{   YwTXEvent Handler, ͿXTƥ                            }
{==============================================================================}
procedure TFallingSlice.Log(Msg: String);
begin
  if Assigned(FOnLog) then FOnLog(Msg);
end;

{==============================================================================}
{   ӲM                                                                   }
{==============================================================================}
procedure TFallingSlice.Reset;
var
  i: Integer;
begin
  FStackHeight := 0;
  for i:=FLoBound to FHiBound do
    FArray[i] := SKY;
end;

{==============================================================================}
{   HLogLXӤe                                                          }
{==============================================================================}
procedure TFallingSlice.Dump;
var
  i:    Integer;
  s:    String;
begin
  s := Format('ID:%d ', [FID]);
  for i:=FLoBound to FHiBound do
    s := s + Format('[%d]=%d ', [i, FArray[i]]);
  s := s + ' StackHi = ' + IntToStr(FStackHeight);
  Log(s);
end;

procedure TFallingSlice.SetFrequency(Value: Byte);
begin
  if (Value > FREQUENCY_MAX) then
    FFrequency := FREQUENCY_MAX
  else if (Value < FREQUENCY_MIN) then
    FFrequency := FREQUENCY_MIN
  else
    FFrequency := Value;
end;

end.
